@@ -48,6 +48,7 @@ class g_:
     signaled = False
     frameDetails = {}
     dataDirPath = None
+    deviceLocsWanted = True
     frameProcessor = None
     framesToDo = collections.deque()
     nFramesWanted = None
@@ -80,13 +81,25 @@ class frameProcessor(object):
 g_.frameProcessor = frameProcessor()
 
 def getInstallerCmd():
-    return g_.frameProcessor.installerCmd()
+    try:
+        return g_.frameProcessor.installerCmd()
+    except Exception as exc:
+        logger.warning( 'the frameProcessor installerCmd() raised exception (%s) %s', type(exc), exc )
+        return None
 
 def getFrameOutFileName( frameNum ):
-    return g_.frameProcessor.frameOutFileName( frameNum )
+    try:
+        return g_.frameProcessor.frameOutFileName( frameNum )
+    except Exception as exc:
+        logger.warning( 'the frameProcessor frameOutFileName() raised exception (%s) %s', type(exc), exc )
+        return None
 
 def getFrameCmd( frameNum ):
-    return g_.frameProcessor.frameCmd( frameNum )
+    try:
+        return g_.frameProcessor.frameCmd( frameNum )
+    except Exception as exc:
+        logger.warning( 'the frameProcessor frameCmd() raised exception (%s) %s', type(exc), exc )
+        return None
 
 class SigTerm(BaseException):
     #logger.warning( 'unsupported SigTerm exception created')
@@ -332,8 +345,20 @@ def recruitInstance( launchedJsonFilePath, resultsLogFilePathIgnored ):
             jsonToKnownHosts.jsonToKnownHosts( startedInstances, khFile )
 
         deadline = min( g_.deadline, time.time() + args.instTimeLimit )
+
+        rc = None
+        if g_.deviceLocsWanted:
+            rc = pushDeviceLoc( inst )
+            if rc:
+                logger.warning( 'rc from pushDeviceLoc was %d', rc )
+                terminateInstances( args.authToken, [iid] )
+                logger.warning( 'terminating instance because pushDeviceLoc %s', iid )
+                terminateInstances( args.authToken, [iid] )
+                logOperation( 'terminateBad', [iid], '<recruitInstances>' )
+                purgeHostKeys( [inst] )
+                return None
         # rsync the common input file, if any
-        if args.commonInFilePath:
+        if args.commonInFilePath and not rc:
             logFrameState( -1, 'rsyncing', iid, 0 )
             destFileName = os.path.basename( args.commonInFilePath )
             (rc, stderr) = rsyncToRemote( args.commonInFilePath, destFileName, inst, timeLimit=args.instTimeLimit )
@@ -489,13 +514,33 @@ def recruitInstances( nWorkersWanted, launchedJsonFilePath, launchWanted, result
         # add instances to knownHosts
         with open( os.path.expanduser('~/.ssh/known_hosts'), 'a' ) as khFile:
             jsonToKnownHosts.jsonToKnownHosts( startedInstances, khFile )
-        # install something on startedInstances, if required, else just return startedInstances
+        
+        if not g_.deviceLocsWanted:
+            goodInstances = startedInstances
+        else:
+            returnCodes = pushDeviceLocs( startedInstances )
+            goodInstances = []
+            badInstances = []
+            for index, rc in enumerate( returnCodes ):
+                if rc == 0:
+                    goodInstances.append( startedInstances[index] )
+                else:
+                    badInstances.append( startedInstances[index] )
+            if badInstances:
+                logger.info( 'could not pushDeviceLoc to %d instances', len(badInstances) )
+                badIids = [inst['instanceId'] for inst in badInstances ]
+                logOperation( 'terminateBad', badIids, '<recruitInstances>' )
+                terminateInstances( args.authToken, badIids )
+                logger.info( 'purging host keys')
+                purgeHostKeys( badInstances )
+
+        # install something on goodInstances, if required, else just return goodInstances
         if (not getInstallerCmd()) and (not args.commonInFilePath):
-            return startedInstances
+            return goodInstances
         if not sigtermSignaled():
             installerCmd = getInstallerCmd()
-            logger.info( 'calling tellInstances to install on %d instances', len(startedInstances))
-            stepStatuses = tellInstances.tellInstances( startedInstances, installerCmd,
+            logger.info( 'calling tellInstances to install on %d instances', len(goodInstances))
+            stepStatuses = tellInstances.tellInstances( goodInstances, installerCmd,
                 resultsLogFilePath=resultsLogFilePath,
                 download=None, downloadDestDir=None, jsonOut=None, sshAgent=args.sshAgent,
                 timeLimit=min(args.instTimeLimit, args.timeLimit), upload=args.commonInFilePath, stopOnSigterm=not True,
@@ -505,7 +550,7 @@ def recruitInstances( nWorkersWanted, launchedJsonFilePath, launchWanted, result
             #signal.signal( signal.SIGTERM, sigtermHandler )
             if not stepStatuses:
                 logger.warning( 'no statuses returned from installer')
-                startedIids = [inst['instanceId'] for inst in startedInstances]
+                startedIids = [inst['instanceId'] for inst in goodInstances]
                 logOperation( 'terminateBad', startedIids, '<recruitInstances>' )
                 terminateInstances( args.authToken, startedIids )
                 return []
@@ -514,14 +559,14 @@ def recruitInstances( nWorkersWanted, launchedJsonFilePath, launchWanted, result
             #eventTimings.append(stepTiming)
             logger.info( '%d good installs, %d bad installs', len(goodOnes), len(badOnes) )
             logger.info( 'stepStatuses %s', stepStatuses )
-            goodInstances = [inst for inst in startedInstances if inst['instanceId'] in goodOnes ]
+            goodInstances = [inst for inst in goodInstances if inst['instanceId'] in goodOnes ]
             badIids = []
             for status in badOnes:
                 badIids.append( status['instanceId'] )
             if badIids:
                 logOperation( 'terminateBad', badIids, '<recruitInstances>' )
                 terminateInstances( args.authToken, badIids )
-                badInstances = [inst for inst in startedInstances if inst['instanceId'] in badIids ]
+                badInstances = [inst for inst in goodInstances if inst['instanceId'] in badIids ]
                 logger.info( 'purging host keys')
                 purgeHostKeys( badInstances )
             #if goodInstances:
@@ -649,6 +694,7 @@ def renderFramesOnInstance( inst ):
     abbrevIid = iid[0:16]
     g_.workingInstances.append( iid )
     saveProgress()
+    logLevel = logger.getEffectiveLevel()
     logger.info( 'would compute frames on instance %s', abbrevIid )
 
     '''
@@ -692,8 +738,9 @@ def renderFramesOnInstance( inst ):
             elif '| Synchronizing object |' in line:
                 pass
             elif line.strip():
-                print( '<stdout>', abbrevIid, line.strip(), file=sys.stderr )
                 logStdout( line.rstrip(), iid )
+                if logLevel <= logging.INFO:
+                    print( '<stdout>', abbrevIid, line.strip(), file=sys.stderr )
     nFailures = 0    
     while len( g_.framesFinished) < g_.nFramesWanted:
         if sigtermSignaled():
@@ -734,81 +781,81 @@ def renderFramesOnInstance( inst ):
 
         outFileName = getFrameOutFileName( frameNum )
         returnCode = None
-        cmd = getFrameCmd( frameNum )
-
-        logger.debug( 'commanding %s', cmd )
-        sshSpecs = inst['ssh']
-
         curFrameRendered = False
-        logFrameState( frameNum, 'starting', iid )
-        frameStartDateTime = datetime.datetime.now(datetime.timezone.utc)
-        with subprocess.Popen(['ssh',
-                            '-p', str(sshSpecs['port']),
-                            '-o', 'ServerAliveInterval=360',
-                            '-o', 'ServerAliveCountMax=3',
-                            sshSpecs['user'] + '@' + sshSpecs['host'], cmd],
-                            encoding='utf8',
-                            stdout=subprocess.PIPE,  # subprocess.PIPE subprocess.DEVNULL
-                            stderr=subprocess.PIPE) as proc:
-            frameProgress = 0
-            frameProgressReported = 0
-            deadline = min( g_.deadline, time.time() + timeLimit )
-            stdoutThr = threading.Thread(target=trackStdout, args=(proc,))
-            stdoutThr.start()
-            stderrThr = threading.Thread(target=trackStderr, args=(proc,))
-            stderrThr.start()
-            while time.time() < deadline:
-                proc.poll() # sets proc.returncode
-                if proc.returncode == None:
-                    if frameProgress > min( .99, frameProgressReported + .01 ):
-                        logger.info( 'frame %d on %s is %.1f %% done', frameNum, abbrevIid, frameProgress*100 )
-                        frameProgressReported = frameProgress
-                        rightNow = datetime.datetime.now(datetime.timezone.utc)
-                        frameDetails[ 'lastDateTime' ] = rightNow.isoformat()
-                        frameDetails[ 'elapsedTime' ] = (rightNow - frameStartDateTime).total_seconds()
-                        frameDetails[ 'progress' ] = frameProgress
-                        saveProgress()
-                    if ((deadline - time.time() < timeLimit/2)) and frameProgress < .5:
-                        #logger.warning( 'frame %d on %s seems slow', frameNum, abbrevIid )
-                        logFrameState( frameNum, 'seemsSlow', iid, frameProgress )
-                else:
-                    if proc.returncode == 0:
-                        logger.info( 'frame %d on %s succeeded', frameNum, abbrevIid )
-                        curFrameRendered = True
-                    else:
-                        logger.warning( 'instance %s gave returnCode %d', abbrevIid, proc.returncode )
-                    break
-                if sigtermSignaled():
-                    break
-                if g_.interrupted:
-                    logger.info( 'exiting polling loop because interrupted' )
-                    break
-                time.sleep(10)
-            returnCode = proc.returncode if proc.returncode != None else 124
-            if returnCode:
-                logger.warning( 'computeFailed with rc %d for frame %d on %s', returnCode, frameNum, iid )
-                logFrameState( frameNum, 'computeFailed', iid, returnCode )
-                frameDetails[ 'progress' ] = 0
-                g_.framesToDo.append( frameNum )
-                saveProgress()
-                time.sleep(10) # maybe we should retire this instance; at least, making it sleep so it is less competitive
-            else:
-                logFrameState( frameNum, 'computed', iid )
-                #g_.framesFinished.append( frameNum )  # too soon
+        cmd = getFrameCmd( frameNum )
+        if cmd:
+            logger.debug( 'commanding %s', cmd )
+            sshSpecs = inst['ssh']
 
-            proc.terminate()
-            try:
-                proc.wait(timeout=5)
-                if proc.returncode:
-                    logger.warning( 'ssh return code %d', proc.returncode )
-            except subprocess.TimeoutExpired:
-                logger.warning( 'ssh did not terminate in time' )
-            stdoutThr.join()
-            stderrThr.join()
+            logFrameState( frameNum, 'starting', iid )
+            frameStartDateTime = datetime.datetime.now(datetime.timezone.utc)
+            with subprocess.Popen(['ssh',
+                                '-p', str(sshSpecs['port']),
+                                '-o', 'ServerAliveInterval=360',
+                                '-o', 'ServerAliveCountMax=3',
+                                sshSpecs['user'] + '@' + sshSpecs['host'], cmd],
+                                encoding='utf8',
+                                stdout=subprocess.PIPE,  # subprocess.PIPE subprocess.DEVNULL
+                                stderr=subprocess.PIPE) as proc:
+                frameProgress = 0
+                frameProgressReported = 0
+                deadline = min( g_.deadline, time.time() + timeLimit )
+                stdoutThr = threading.Thread(target=trackStdout, args=(proc,))
+                stdoutThr.start()
+                stderrThr = threading.Thread(target=trackStderr, args=(proc,))
+                stderrThr.start()
+                while time.time() < deadline:
+                    proc.poll() # sets proc.returncode
+                    if proc.returncode == None:
+                        if frameProgress > min( .99, frameProgressReported + .01 ):
+                            logger.info( 'frame %d on %s is %.1f %% done', frameNum, abbrevIid, frameProgress*100 )
+                            frameProgressReported = frameProgress
+                            rightNow = datetime.datetime.now(datetime.timezone.utc)
+                            frameDetails[ 'lastDateTime' ] = rightNow.isoformat()
+                            frameDetails[ 'elapsedTime' ] = (rightNow - frameStartDateTime).total_seconds()
+                            frameDetails[ 'progress' ] = frameProgress
+                            saveProgress()
+                        if ((deadline - time.time() < timeLimit/2)) and frameProgress < .5:
+                            #logger.warning( 'frame %d on %s seems slow', frameNum, abbrevIid )
+                            logFrameState( frameNum, 'seemsSlow', iid, frameProgress )
+                    else:
+                        if proc.returncode == 0:
+                            logger.info( 'frame %d on %s succeeded', frameNum, abbrevIid )
+                            curFrameRendered = True
+                        else:
+                            logger.warning( 'instance %s gave returnCode %d', abbrevIid, proc.returncode )
+                        break
+                    if sigtermSignaled():
+                        break
+                    if g_.interrupted:
+                        logger.info( 'exiting polling loop because interrupted' )
+                        break
+                    time.sleep(10)
+                returnCode = proc.returncode if proc.returncode != None else 124
+                if returnCode:
+                    logger.warning( 'computeFailed with rc %d for frame %d on %s', returnCode, frameNum, iid )
+                    logFrameState( frameNum, 'computeFailed', iid, returnCode )
+                    frameDetails[ 'progress' ] = 0
+                    g_.framesToDo.append( frameNum )
+                    saveProgress()
+                    time.sleep(10) # maybe we should retire this instance; at least, making it sleep so it is less competitive
+                else:
+                    logFrameState( frameNum, 'computed', iid )
+                    #g_.framesFinished.append( frameNum )  # too soon
+
+                proc.terminate()
+                try:
+                    proc.wait(timeout=5)
+                    if proc.returncode:
+                        logger.warning( 'ssh return code %d', proc.returncode )
+                except subprocess.TimeoutExpired:
+                    logger.warning( 'ssh did not terminate in time' )
+                stdoutThr.join()
+                stderrThr.join()
         # may not need this logging here
-        if returnCode != 0:
-            logger.warning( 'remote returnCode %d for %s', returnCode, abbrevIid )
-        if curFrameRendered:
+        #if returnCode != 0:
+        #    logger.warning( 'remote returnCode %d for %s', returnCode, abbrevIid )
+        if curFrameRendered and outFileName:
             logFrameState( frameNum, 'retrieving', iid )
             (returnCode, stderr) = scpFromRemote( 
                 outFileName, g_.dataDirPath, inst
@@ -840,6 +887,101 @@ def renderFramesOnInstance( inst ):
         saveProgress()
     return 0
 
+def commandInstance( inst, cmd, timeLimit ):
+    deadline = time.time() + timeLimit
+    sshSpecs = inst['ssh']
+    #logInstallerOperation( iid, ['connect', sshSpecs['host'], sshSpecs['port']] )
+    with subprocess.Popen(['ssh',
+                    '-p', str(sshSpecs['port']),
+                    '-o', 'ServerAliveInterval=360',
+                    '-o', 'ServerAliveCountMax=3',
+                    sshSpecs['user'] + '@' + sshSpecs['host'], cmd],
+                    encoding='utf8',
+                    #stdout=subprocess.PIPE,  # subprocess.PIPE subprocess.DEVNULL
+                    ) as proc:  # stderr=subprocess.PIPE
+        #logInstallerOperation( iid, ['command', cmd] )
+        #stderrThr = threading.Thread(target=trackStderr, args=(proc,))
+        #stderrThr.start()
+        abbrevIid = inst['instanceId'][0:16]
+        while time.time() < deadline:
+            proc.poll() # sets proc.returncode
+            if proc.returncode == None:
+                logger.info( 'waiting for command on instance %s', abbrevIid)
+            else:
+                if proc.returncode == 0:
+                    logger.debug( 'command succeeded on instance %s', abbrevIid )
+                else:
+                    logger.warning( 'instance %s gave returnCode %d', abbrevIid, proc.returncode )
+                break
+            if sigtermSignaled():
+                break
+            if g_.interrupted:
+                break
+            time.sleep(5)
+        proc.poll()
+        returnCode = proc.returncode if proc.returncode != None else 124 # declare timeout if no rc
+        if returnCode:
+            logger.warning( 'installer returnCode %s', returnCode )
+        #if returnCode == 124:
+        #    logInstallerEvent( 'timeout', args.instTimeLimit, iid )
+        #else:
+        #    logInstallerEvent('returncode', returnCode, iid )
+        proc.terminate()
+        try:
+            proc.wait(timeout=5)
+            if proc.returncode:
+                logger.warning( 'ssh return code %d', proc.returncode )
+        except subprocess.TimeoutExpired:
+            logger.warning( 'ssh did not terminate in time' )
+        #stderrThr.join()
+        if returnCode:
+            #logger.warning( 'terminating instance because installerFailed %s', iid )
+            #terminateInstances( args.authToken, [iid] )
+            #logOperation( 'terminateBad', [iid], '<recruitInstances>' )
+            #purgeHostKeys( [inst] )
+            return returnCode
+        else:
+            return 0
+    return 1
+
+def pushDeviceLoc( inst, timeLimit=30 ):
+    iid = inst['instanceId']
+    logger.debug( 'would push deviceLoc to instance %s', iid[0:16] )
+    rc = 1
+    if not inst.get( 'device-location' ):
+        logger.warning( 'no device-location for %s', iid[0:16] )
+    else:
+        logFrameState( -1, 'pushDeviceLocStarting', iid )
+        deviceLoc = inst['device-location']
+        deviceLocJson = json.dumps( deviceLoc )
+        # trickily enquote and embedded apostrophes (single-quotes) for bash compatibility
+        deviceLocJson = deviceLocJson.replace( "'", r"'\''")
+        # generate a command to create the json file on the instance
+        cmd = "mkdir ~/.neocortix && echo '%s' > ~/.neocortix/device-location.json" % deviceLocJson
+        logger.debug( 'cmd: %s', cmd )
+        rc = commandInstance( inst, cmd, timeLimit=timeLimit )
+        logFrameState( -1, 'pushDeviceLocDone', iid, rc )
+    return rc
+
+def pushDeviceLocs( instances, timeLimit=40 ):
+    '''push device-location info to instances, in parallel'''
+    returnCodes = []
+    with futures.ThreadPoolExecutor( max_workers=len(instances) ) as executor:
+        parIter = executor.map( pushDeviceLoc, instances, timeout=timeLimit )
+        returnCodes = [None] * len(instances)
+        try:
+            index = 0
+            for returnCode in parIter:
+                returnCodes[index] = returnCode
+                index += 1
+                time.sleep( .1 )
+        except KeyboardInterrupt:
+            logger.warning( 'interrupted, setting flag')
+            g_.interrupted = True
+            raise
+        logger.debug( 'returnCodes: %s', returnCodes )
+    return returnCodes
+
 def recruitAndRender():
     '''a threadproc to recruit an instance, compute frames on it, and terminate it'''
     eLoop = asyncio.new_event_loop()
@@ -970,6 +1112,9 @@ def runBatch( **kwargs ):
     if resp['statusCode'] == 403:
         logger.error( 'the given authToken was not accepted' )
         return 1
+    elif resp['statusCode'] not in range( 200, 300 ):
+        logger.error( 'service error (%d) while validating authToken' )
+        return 1
     g_.limitOneFramePerWorker = args.limitOneFramePerWorker
     g_.autoscaleMax = 1
     if not args.nWorkers:
@@ -1101,7 +1246,7 @@ def createArgumentParser():
         fromfile_prefix_chars='@', formatter_class=argparse.ArgumentDefaultsHelpFormatter )
     ap.add_argument( '--commonInFilePath', help='a file to upload initially to all instances' )
     ap.add_argument( '--authToken', required=True, help='the NCS authorization token to use (required)' )
-    ap.add_argument( '--outDataDir', help='output data darectory', default='./aniData/' )
+    ap.add_argument( '--outDataDir', help='output data darectory', default='./data/' )
     ap.add_argument( '--encryptFiles', type=boolArg, default=True, help='whether to encrypt files on launched instances' )
     ap.add_argument( '--filter', help='json to filter instances for launch' )
     ap.add_argument( '--frameTimeLimit', type=int, default=8*60*60, help='amount of time (in seconds) allowed for each frame' )
